@@ -1,4 +1,14 @@
-`websocket_write` <- function(DATA, WS)
+daemonize = function(server)
+{
+  if(.Platform$OS.type != "unix") stop("Sorry, daemonize requires a unix-type operating system.\nUse service in an explicit event loop instead.")
+  if(!exists("server_list",envir=.websockets_env))
+    assign("server_list",c(),envir=.websockets_env)
+  server$handler = .register_event_handler(server$server_socket)
+  .websockets_env$server_list = c(.websockets_env$server_list,server)
+}
+
+# Write data to a websocket
+websocket_write = function(DATA, WS)
 {
   mask = FALSE
   if(is.null(WS$server)) {
@@ -6,7 +16,7 @@
     mask = TRUE
     WS = WS$client_sockets[[1]]
   }
-  v <- WS$wsinfo$v
+  v = WS$wsinfo$v
 # Give up silently. I supressed the warning since broadcast might
 # easily hit a non-websocket client (say, a web page request).
   if(is.null(v)) {
@@ -16,19 +26,23 @@
   if(is.character(DATA)) DATA=charToRaw(DATA)
   if(!is.raw(DATA)) stop("DATA must be character or raw")
   if(v==4){
-    j <-.SOCK_SEND(WS$socket,.frame(length(DATA),mask=mask))
+    j =.SOCK_SEND(WS$socket,.frame(length(DATA),mask=mask,
+                                    opcode=(if (WS$server$is.binary) { 2L } else { 1L })))
     if(j<0) {
       websocket_close(WS)
       return(j)
     }
     if(mask) {
       key = as.raw(floor(runif(4)*256))
-      j <- .SOCK_SEND(WS$socket, key)
+      j = .SOCK_SEND(WS$socket, key)
       return(.SOCK_SEND(WS$socket, .MASK(DATA,key)))
     }
     return(.SOCK_SEND(WS$socket, DATA))
   }
-  j <- .SOCK_SEND(WS$socket,raw(1))
+  if (WS$server$is.binary)
+    j = .SOCK_SEND(WS$socket,raw(2))
+  else
+    j = .SOCK_SEND(WS$socket,raw(1))
   if(j<0) {
     websocket_close(WS)
     return(j)
@@ -37,13 +51,14 @@
   .SOCK_SEND(WS$socket,packBits(intToBits(255))[1])
 }
 
-`websocket_broadcast` <- function(DATA, server)
+# "Broadcast" data to all the client websockets attached to the server.
+websocket_broadcast = function(DATA, server)
 {
   lapply(server$client_sockets, function(x) websocket_write(DATA,x))
 }
 
-`set_callback` <- function(id, f, envir) assign(id, f, envir=envir)
-`setCallback` <- function(id, f, envir)
+set_callback = function(id, f, envir) assign(id, f, envir=envir)
+setCallback = function(id, f, envir)
 {
   assign(id, f, envir=envir)
 }
@@ -51,16 +66,16 @@
 # Example static service function closure:
 # Supply any function that takes (socket, header) args
 # to handle request...
-`static_file_service` <- function(fn)
+static_file_service = function(fn)
 {
-  file_name <- fn
-  f <- file(fn)
-  file_content <- paste(readLines(f),collapse="\n")
+  file_name = fn
+  f = file(fn)
+  file_content = paste(readLines(f),collapse="\n")
   close(f)
   function(socket, header) {
-    finf <- file.info(fn)
+    finf = file.info(fn)
     if(difftime(finf[,"mtime"],Sys.time(),units="secs")>0){
-      f <- file(fn)
+      f = file(fn)
       file_content <<- paste(readLines(f),collapse="\n")
       close(f)
     }
@@ -71,12 +86,13 @@
     else {
       .http_200(socket,content=file_content)
     }
+    return(TRUE)
   }
 }
 
 # A simpler example web-page service that serves up static
 # text (and an icon).
-`static_text_service` <- function(text)
+static_text_service = function(text)
 {
   function(socket, header) {
     if(is.null(header$RESOURCE)) return(.http_400(socket))
@@ -86,28 +102,31 @@
     else {
       .http_200(socket,content=text)
     }
+    return(TRUE)
   }
 }
 
 # Will eventually switch naming convention, for now this is a doppelganger.
-`create_server` <- function(
+create_server = function(
       port=7681L,
       webpage=static_file_service(
-        paste(system.file(package='websockets'), "basic.html",sep="//")))
+        paste(system.file(package='websockets'), "basic.html",sep="//")),
+      is.binary=FALSE)
 {
-  createContext(port, webpage)
+  createContext(port, webpage, is.binary=is.binary)
 }
 
 # A server (formerly context) is an environment that stores data associated
 # with a single websocket server port, including information on all clients
-# connected to that server, and a function that serves up static web pages.
-`createContext` <- function(
+# connected to that server, and a function that returns static web pages.
+`createContext` = function(
       port=7681L,
       webpage=static_file_service(
         paste(system.file(package='websockets'), "basic.html",sep="//")),
-      server=TRUE)
+      server=TRUE,
+      is.binary=FALSE)
 {
-  w <- new.env()
+  w = new.env()
   assign('static', webpage, envir=w)
 # server_socket is the file descriptor associated with this server
   if(server)
@@ -120,29 +139,39 @@
 # wsinfo  (parsed websocket header data -- another list)
 # server  (the environment associated with the server for this client)
   assign('client_sockets', list(), envir=w)
-# This is not required, but we supply a default recieve function:
+# This is not required, but we supply a default websocket recieve function:
   assign('receive', function(WS, DATA, HEADER=NULL) {
                       cat("Received data from client socket ",WS$socket,":\n")
                       if(is.raw(DATA)) cat(rawToChar(DATA),"\n")
                     },envir=w)
+# A default websocket close function:
   assign('closed', function(WS) {
                       cat("Client socket",WS$socket," was closed.\n")
                       if(is.null(WS$wsinfo)){cat("(It was not a websocket client, just a static web page.)\n")}
                    },envir=w)
+# A default websocket established function:
   assign('established', function(WS) {
                       cat("Client socket",WS$socket," has been established.\n")
                    },envir=w)
+  assign('is.binary', is.binary, envir=w)
+  reg.finalizer(w, function(e) {websocket_close(e)})
   return(w)
 }
 
  
-# Cleanly close a websocket client connection or server
-`websocket_close` <- function(connection)
+# Cleanly close any websocket connection (client or server)
+`websocket_close` = function(connection)
 {
   if(!is.null(connection$socket)) .remove_client(connection)
   else {
 # This is not a client socket, perhaps a server?
     for(j in connection$client_sockets) .remove_client(j)
+    if(!is.null(connection$handler))
+    {
+      .deregister_event_handler(connection$handler)
+      connection$handler = NULL
+      .undaemon(connection)
+    }
     if(!is.null(connection$server_socket))
       .SOCK_CLOSE(connection$server_socket)
   }
@@ -152,12 +181,12 @@
 # Naming convention may change in a futer version: 'context' will be
 # replaced by 'server.' Both are present in this version for compatibility
 # with old package versions.
-`service` <- function(context, timeout=1000L, server=context)
+`service` = function(context, timeout=1000L, server=context)
 {
-  socks <- c(server$server_socket,
+  socks = c(server$server_socket,
     unlist(lapply(server$client_sockets,function(x) x$socket)))
   if(length(socks)<1) return(invisible())
-  s <- .SOCK_POLL(socks, timeout=timeout)
+  s = .SOCK_POLL(socks, timeout=timeout)
   for(j in s){
     if(j==server$server_socket){
 # New client connection
@@ -166,8 +195,8 @@
     else{
 # j holds just the socket file descriptor, or a negated descriptor
 # indicating an error condition. Retrieve the client socket from the server
-# environment in J. XXX Improve this with a hashed lookup.
-      J <- server$client_sockets[[as.character(j)]]
+# environment in J.
+      J = server$client_sockets[[as.character(abs(j))]]
       if(j<0) {
 # Poll reports an error condition for this socket. Close it.
         websocket_close(J)
@@ -176,29 +205,33 @@
 # A connected client is sending us something!
       if(J$new) {
 # This is a new client connection, handshake.
-        J$new <- FALSE
-        x <- .SOCK_RECV_HTTP_HEAD(j)
-        h <- .parse_header(x)
+        J$new = FALSE
+        x = .SOCK_RECV_HTTP_HEAD(j)
+        h = .parse_header(x)
         if(is.null(h)) {
 # Shucks, something wrong with this client. Drop him.
           websocket_close(J)
           next
         }
-        v <- 0
+        v = 0
         if(!is.null(h[["Sec-WebSocket-Version"]]))
-          if(as.numeric(h[["Sec-WebSocket-Version"]])>=4) v <- 4
-        h$v <- v
+          if(as.numeric(h[["Sec-WebSocket-Version"]])>=4) v = 4
+        h$v = v
 # Stash this client's header, identifying websocket protocol version, etc.
 # in the appropriate client_socket list
-        cs <- server$client_sockets
-        J$wsinfo <- h
-        cs[[as.character(j)]] <- J
+        cs = server$client_sockets
+        J$wsinfo = h
+        cs[[as.character(j)]] = J
         assign("client_sockets",cs,envir=server)
 
         if(is.null(h$Upgrade)) {
 # Not a handshake request, serve a static web page
-          if(is.function(server$static)) server$static(j,h)
-          .remove_client(J)
+          if(is.function(server$static)) {
+# 
+# NOTE: CONNECTION REMAINS OPEN IF STATIC WEB SERVICE DOES NOT RETURN TRUE
+# We allow this case to pass connection on to ancillary service, for example. 
+            if(server$static(j,h)) .remove_client(J)
+          } else .remove_client(J)
           next
         }
 # Negotiate a websocket connection
@@ -210,10 +243,10 @@
         next
       } else if(J$wsinfo$v < 4) {
 # Old protocol
-        x <- .SOCK_RECV_FRAME00(j,max_buffer_size=getOption("websockets_max_buffer_size"))
+        x = .SOCK_RECV_FRAME00(j,max_buffer_size=getOption("websockets_max_buffer_size"))
       } else {
 # Try the latest protocol
-        x <- .SOCK_RECV_FRAME(j,max_buffer_size=getOption("websockets_max_buffer_size"))
+        x = .SOCK_RECV_FRAME(j,max_buffer_size=getOption("websockets_max_buffer_size"))
       }
       if(length(x)<1) {
 # Can't have an empty transmission, close the socket.
@@ -226,7 +259,7 @@
         server$receive(WS=J, DATA=.v00_unframe(x), HEADER=NULL)
       }
       else{
-        DATA <- .unframe(x)
+        DATA = .unframe(x)
         if(DATA$header$opcode < 3){
           server$receive(WS=J, DATA=DATA$data, HEADER=DATA$header)
         } else if(DATA$header$opcode == 8) {
@@ -324,7 +357,7 @@
     .SOCK_CLOSE(s)
     stop("Connection timeout")
   }
-  x <- .SOCK_RECV_HTTP_HEAD(s)
+  x = .SOCK_RECV_HTTP_HEAD(s)
   if(length(x)<12) stop("Connection error")
   if(!all(x[1:12] == charToRaw("HTTP/1.1 101")))
     stop(paste("Connection error: ",rawToChar(x),sep="\n"))
@@ -333,9 +366,48 @@
 # XXX XXX
   if(version==0) {
     if(.SOCK_POLL(s)>0)
-      x <- .SOCK_RECV(s, buf_size=16L, max_buffer_size=16)
+      x = .SOCK_RECV(s, buf_size=16L, max_buffer_size=16)
   }
-  context$client_sockets[[as.character(s)]] <- 
+  context$client_sockets[[as.character(s)]] = 
     list(socket=s, wsinfo=list(v=version), server=context, new=FALSE)
   context
 }
+
+# Parse http get/post variables, returning a list.
+# We don't really handle the POST case. Use a better web server
+# for that!
+http_vars = function(socket, header)
+{
+  res = strsplit(header$RESOURCE,split="\\?")[[1]]
+  if(header$TYPE=="POST") {
+    N = header[["Content-Length"]]
+    if(!is.null(N)) return(rawToChar(.SOCK_RECV_N(socket, N)))
+    else stop("Error in POST handling")
+  }
+  else GET = res[2]
+  if(!is.na(GET) && nchar(GET)>1) {
+    GET = lapply(strsplit(GET,"&")[[1]],function(x) strsplit(x,"=")[[1]])
+    gnams = lapply(GET,function(x) x[[1]])
+    GET = lapply(GET,function(x) if(length(x)>1){.urldecode(x[[2]])} else{c()})
+    names(GET) = gnams
+  } else GET = c()
+  GET
+}
+
+# A basic and generic http response function
+http_response = function(socket, status=200,
+                         content_type="text/html; charset=UTF-8", content="")
+{
+  n = ifelse(is.character(content),nchar(content), length(content))
+  h=paste("HTTP/1.1",status,"OK\r\nServer: R/Websocket\r\n")
+  h=paste(h,"Content-Type: ",content_type, "\r\n",sep="")
+  h=paste(h,"Date: ",date(),"\r\n",sep="")
+  h=paste(h,"Content-Length: ",n,"\r\n\r\n",sep="")
+  .SOCK_SEND(socket,charToRaw(h))
+  if(is.character(content))
+    .SOCK_SEND(socket,charToRaw(content))
+  else
+    .SOCK_SEND(socket,content)
+  .SOCK_CLOSE(socket)
+  TRUE
+}
